"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const node_1 = require("./node");
const common_spec_1 = require("./common.spec");
describe('popsicle node', () => {
    it('should return 2xx statuses', () => __awaiter(this, void 0, void 0, function* () {
        const res = yield node_1.transport()(node_1.request(`${common_spec_1.TEST_HTTP_URL}/status/204`));
        expect(res.ok).toEqual(true);
        expect(res.statusCode).toEqual(204);
        expect(res.statusMessage).toEqual('No Content');
    }));
    it('should return 4xx statuses', () => __awaiter(this, void 0, void 0, function* () {
        const res = yield node_1.transport()(node_1.request(`${common_spec_1.TEST_HTTP_URL}/status/404`));
        expect(res.ok).toEqual(false);
        expect(res.statusCode).toEqual(404);
        expect(res.statusMessage).toEqual('Not Found');
    }));
    it('should return 5xx statuses', () => __awaiter(this, void 0, void 0, function* () {
        const res = yield node_1.transport()(node_1.request(`${common_spec_1.TEST_HTTP_URL}/status/500`));
        expect(res.ok).toEqual(false);
        expect(res.statusCode).toEqual(500);
        expect(res.statusMessage).toEqual('Internal Server Error');
    }));
    it('should always send user agent', () => __awaiter(this, void 0, void 0, function* () {
        const res = yield node_1.transport()(node_1.request(`${common_spec_1.TEST_HTTP_URL}/echo/header/user-agent`));
        expect(yield res.body.text()).toEqual('Popsicle (https://github.com/serviejs/popsicle)');
    }));
    it('send a custom user agent header', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/echo/header/user-agent`, {
            headers: { 'User-Agent': 'foobar' }
        });
        const res = yield node_1.transport()(req);
        expect(yield res.body.text()).toEqual('foobar');
    }));
    it('should send case sensitive headers', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/raw-headers`, {
            headers: { 'RaW-HeAder': 'test' }
        });
        const res = yield node_1.transport()(req);
        expect(yield res.body.json()).toContain('RaW-HeAder');
    }));
    it('should send post data', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/echo`, {
            method: 'POST',
            body: 'example data',
            headers: {
                'content-type': 'application/octet-stream'
            }
        });
        const res = yield node_1.transport()(req);
        expect(res.statusCode).toEqual(200);
        expect(res.statusMessage).toEqual('OK');
        expect(res.headers.get('Content-Type')).toEqual('application/octet-stream');
        expect(yield res.body.text()).toEqual('example data');
    }));
    it('should abort before it starts', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/echo`);
        req.abort();
        expect.assertions(2);
        try {
            yield node_1.transport()(req);
        }
        catch (err) {
            expect(err.message).toEqual('Request aborted');
            expect(err.code).toEqual('EABORT');
        }
    }));
    it('should abort mid-request', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/download`);
        const res = yield node_1.transport()(req);
        setTimeout(() => req.abort(), 100);
        expect(yield res.body.text()).toEqual('hello ');
    }));
    it('should have no side effects aborting twice', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/download`);
        expect.assertions(1);
        req.abort();
        req.abort();
        try {
            const res = yield node_1.transport()(req);
            expect(yield res.body.text()).toEqual('');
        }
        catch (err) {
            expect(err.code).toEqual('EABORT');
        }
    }));
    it('should emit download progress', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/download`);
        const spy = jest.fn();
        const res = yield node_1.transport()(req);
        res.events.on('progress', spy);
        expect(yield res.body.text()).toEqual('hello world!');
        // Check spy after body has loaded.
        expect(spy).toBeCalledWith(12);
    }));
    it('should work with a cookie jar', () => __awaiter(this, void 0, void 0, function* () {
        let cookie;
        const t = node_1.transport({ jar: node_1.cookieJar() });
        const cookieRes = yield t(node_1.request(`${common_spec_1.TEST_HTTP_URL}/cookie`));
        expect(cookieRes.headers.has('Cookie')).toEqual(false);
        expect(cookieRes.headers.has('Set-Cookie')).toEqual(true);
        cookie = (cookieRes.headers.get('Set-Cookie') || '').split(';').shift();
        const echoRes = yield t(node_1.request(`${common_spec_1.TEST_HTTP_URL}/echo`));
        expect(echoRes.headers.get('Cookie')).toEqual(cookie);
        expect(echoRes.headers.has('Set-Cookie')).toEqual(false);
    }));
    it('should update cookies over redirects', () => __awaiter(this, void 0, void 0, function* () {
        const t = node_1.transport({ jar: node_1.cookieJar() });
        const res = yield t(node_1.request(`${common_spec_1.TEST_HTTP_URL}/cookie/redirect`));
        expect(yield res.body.text()).toMatch(/^new=cookie/);
    }));
    it('should follow 302 redirect with get', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect`);
        const spy = jest.fn();
        req.events.on('redirect', spy);
        const res = yield node_1.transport()(req);
        expect(spy).toBeCalledWith(`${common_spec_1.TEST_HTTP_URL}/destination`);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('welcome get');
    }));
    it('should follow 301 redirect with post', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/301`, { method: 'post' });
        const res = yield node_1.transport()(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('welcome get');
    }));
    it('should follow 303 redirect with post', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/303`, { method: 'post' });
        const res = yield node_1.transport()(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('welcome get');
    }));
    it('should support disabling following redirects', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect`, { method: 'post' });
        const res = yield node_1.transport({ follow: false })(req);
        expect(res.statusCode).toEqual(302);
        expect(yield res.body.text()).toEqual('Found. Redirecting to /destination');
    }));
    it('should default to a maximum of 5 redirects', () => __awaiter(this, void 0, void 0, function* () {
        expect.assertions(2);
        try {
            yield node_1.transport()(node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/6`));
        }
        catch (err) {
            expect(err.code).toEqual('EMAXREDIRECTS');
            expect(err.message).toEqual('Maximum redirects exceeded: 5');
        }
    }));
    it('should change maximum redirects', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/6`);
        const res = yield node_1.transport({ maxRedirects: 10 })(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('welcome get');
    }));
    it('should support head redirects with 307', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/307`, { method: 'head' });
        const res = yield node_1.transport()(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('');
    }));
    it('should block 307/308 redirects by default', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/307`, { method: 'post' });
        const res = yield node_1.transport()(req);
        expect(res.statusCode).toEqual(307);
        expect(yield res.body.text()).toEqual('Temporary Redirect. Redirecting to /destination');
    }));
    it('should stop redirecting on false confirmation', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/307`, { method: 'post' });
        const confirmRedirect = jest.fn(() => false);
        const res = yield node_1.transport({ confirmRedirect })(req);
        expect(confirmRedirect).toHaveBeenCalledTimes(1);
        expect(res.statusCode).toEqual(307);
        expect(yield res.body.text()).toEqual('Temporary Redirect. Redirecting to /destination');
    }));
    it('should support user confirmed redirects with 308', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(`${common_spec_1.TEST_HTTP_URL}/redirect/status/307`, { method: 'POST' });
        const confirmRedirect = jest.fn(() => true);
        const res = yield node_1.transport({ confirmRedirect })(req);
        expect(confirmRedirect).toHaveBeenCalledTimes(1);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('welcome post');
    }));
    it('should reject https unauthorized', () => __awaiter(this, void 0, void 0, function* () {
        expect.assertions(1);
        try {
            yield node_1.transport()(node_1.request(common_spec_1.TEST_HTTPS_URL));
        }
        catch (err) {
            expect(err.code).toEqual('EUNAVAILABLE');
        }
    }));
    it('should support https ca option', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(common_spec_1.TEST_HTTPS_URL);
        const ca = fs_1.readFileSync(path_1.join(__dirname, '../scripts/support/ca-crt.pem'));
        const res = yield node_1.transport({ ca })(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('Success');
    }));
    it('should support disabling reject unauthorized', () => __awaiter(this, void 0, void 0, function* () {
        const req = node_1.request(common_spec_1.TEST_HTTPS_URL);
        const res = yield node_1.transport({ rejectUnauthorized: false })(req);
        expect(res.statusCode).toEqual(200);
        expect(yield res.body.text()).toEqual('Success');
    }));
});
//# sourceMappingURL=node.spec.js.map